/*
 * Sieve.java
 *
 * Alternate version of Sieve of Eratosthenes.
 * Written by John M. Hunt (John.Hunt@covenant.edu).
 *
 * Keeps an array of booleans indicating if each value is a prime.
 */

public class Sieve {
	public static void printSieve(boolean a[]) {
		for (int i = 2; i < a.length; i++) {
			if (a[i] == true) {
				System.out.print(i + ", ");
			}
		}
	}

	public static int countSieve(boolean a[]) {
		int count = 0;
		for (int i = 2; i < a.length; i++) {
			if (a[i] == true) {
				count++;
			}
		}
		return count;
	}

	/*
	 * For each identified prime (true value), cross out all of its multiples
	 */
	public static void runSieve(boolean a[]) {
		for (int i = 2; i < a.length; i++) {
			if (a[i] == true) {
				for (int n = i + i; n < a.length; n = n + i) {
					a[n] = false;
				}
			}
		}
	}

	public static boolean[] initSieve(int size) {
		boolean nums[] = new boolean[size];
		for (int i = 0; i < nums.length; i++) {
			nums[i] = true;
		}
		return nums;
	}

	public static void main(String args[]) {
		long startMillis = System.currentTimeMillis();

		int size = 8000000;
		boolean a[] = initSieve(size);
		runSieve(a);
		System.out.println("number of primes = " + countSieve(a));
		// printSieve(a);
		long endMillis = System.currentTimeMillis();
		double durationSecs = ((double) (endMillis - startMillis)) / 1000d;
		System.out.println("elapsed time in seconds: " + durationSecs);
	}
}